--[[
Copyright (c) 2012-2013 Daniel Hagerstrand

This software is provided 'as-is', without any express or implied warranty. In
no event will the authors be held liable for any damages arising from the use
of this software.

Permission is granted to anyone to use this software for any purpose, including
commercial applications, and to alter it and redistribute it freely, subject to
the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software in a
product, an acknowledgment in the product documentation would be appreciated
but is not required.

    2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.

    3. This notice may not be removed or altered from any source distribution.
--]]


--[[
Costume - A set of animation images that an Entity can "wear" to set its
appearance.

Loading a Costume will automatically load all Animations it contains.
--]]


require "lib.klutils.cabin"
require "lib.klutils.oops"

-- Private fields go here
Costume = BaseObj:makeClass{
	_name = nil,
	_animations = nil
}

function Costume:init(name)
	if not name then
		return
	end
	
	if not self._animations then
		self._animations = {}
	end
	
	-- Folder where we will get the costumes from.
	local filepath = "data/costumes/" .. name
	
	if not love.filesystem.isDirectory(filepath) then
		cabin.error("Could not open costume \"" .. name .. "\". Does not exist.")
	end
	
	local fileTable = love.filesystem.enumerate(filepath)
	
	local index = 1
	self._frames = {}
	
	for i, v in ipairs(fileTable) do
		local filename = filepath .. "/" .. v
		
		if love.filesystem.isDirectory(filename) and string.sub(v, 1, 1) ~= "." then
			local newAnimation = Animation:new(name, v)
			self._animations[v] = newAnimation
		end
	end

	self._numFrames = index - 1
	self._name = name
	
	-- Try to use the costume's root folder as the idle animation if one is not provided.
	if not self._animations["idle"] then
		local idleAnim = Animation:new(name)
		
		if idleAnim:getFramesCount() <= 0 then
			cabin.warning("No \"idle\" animation provided for costume " .. name .. ". Costume will be invisible.")
		end
		
		self._animations["idle"] = idleAnim
	end
end

-- Getters --

function Costume:getName()
	return self._name
end

function Costume:getAnimation(name)
	if self._animations then
		return self._animations[name]
	end
end

-- Extra Methods --

function Costume:hasAnimation(name)
	if self._animations and self._animations[name] then
		return true
	end
	
	return false
end
