--[[
Copyright (c) 2012-2013 Daniel Hagerstrand

This software is provided 'as-is', without any express or implied warranty. In
no event will the authors be held liable for any damages arising from the use
of this software.

Permission is granted to anyone to use this software for any purpose, including
commercial applications, and to alter it and redistribute it freely, subject to
the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software in a
product, an acknowledgment in the product documentation would be appreciated
but is not required.

    2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.

    3. This notice may not be removed or altered from any source distribution.
--]]


--[[
TextEntity - A superclass of Entity with its draw() method overloaded to draw
text instead of sprites.
--]]


require "lib.klutils.oops"

-- Private fields go here
TextEntity = Entity:makeClass{
	_static = {
		_fontsCache = {}
	},
	_string = ""
}

-- TODO: Implement font cache cleanup? They don't seem to take up a lot of RAM
--  but still might be a good idea.

function TextEntity:init(x, y, string, fontFace, fontSize)
	Entity.init(self, x, y)
	
	if string then
		self:setString(string)
	end
	if fontFace and fontSize then
		self:setFont(fontFace, fontSize)
	end
end

-- Getters --

function TextEntity:getString()
	return self._string
end

-- Setters --

function TextEntity:setFont(newFont, fontSize)
	if newFont then
		-- Create a font size table for this font if it doesn't yet exist
		if not self._static._fontsCache[newFont] then
			self._static._fontsCache[newFont] = {}
		end
		-- Check font face and font size in cache
		if self._static._fontsCache[newFont][fontSize] then
			self._font = self._static._fontsCache[newFont][fontSize]
		else
			local font = love.graphics.newFont("data/fonts/" .. newFont .. ".ttf", fontSize)

			self._static._fontsCache[newFont][fontSize] = font
			self._font = font
		end
	end
end

function TextEntity:setString(newString)
	if newString then
		self._string = newString
	end
end

-- Extra Methods --

function TextEntity:setBounds(width, alignment)
	if width then
		self._textWidth = width
		self._alignment = alignment or "left"
	end
end

function TextEntity:removeBounds()
	self._textWidth = nil
	self._alignment = nil
end

function TextEntity:draw()
	if self._font then
		love.graphics.setFont(self._font)
		love.graphics.setColor(self._color)
		if self._textWidth then
			love.graphics.printf(self._string, self._x, self._y, self._textWidth, self._alignment)
		else
			love.graphics.print(self._string, self._x, self._y)
		end
	end
end

