--[[
Copyright (c) 2013 Daniel Hagerstrand

This software is provided 'as-is', without any express or implied warranty. In
no event will the authors be held liable for any damages arising from the use
of this software.

Permission is granted to anyone to use this software for any purpose, including
commercial applications, and to alter it and redistribute it freely, subject to
the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software in a
product, an acknowledgment in the product documentation would be appreciated
but is not required.

    2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.

    3. This notice may not be removed or altered from any source distribution.
--]]


--[[
TilegridEntity - an entity that represents a movable tilegrid.
--]]


require "lib.klutils.cabin"
require "lib.klutils.objectlist"
require "lib.klutils.oops"

require "lib.klua.entity"

-- Private fields go here
TilegridEntity = Entity:makeClass
{
	_tilegrid = nil
}

function TilegridEntity:init(x, y, tilegrid)
	self._tilegrid = tilegrid
	
	Entity.init(self, x, y)
end

-- Getters --

function TilegridEntity:getTilegrid()
	return self._tilegrid
end

-- Setters --

function TilegridEntity:setTilegrid(tilegrid)
	self._tilegrid = tilegrid
end

-- Extra Methods --

function TilegridEntity:draw()
	if self._tilegrid then
		local tileset = self._tilegrid:getTileset()
		
		if tileset then
			love.graphics.setColor(self:getColor())
			
			love.graphics.push()
			
			local totalWidth, totalHeight = self._tilegrid:getSize()
			totalWidth = totalWidth * tileset:getTileSize()
			totalHeight = totalHeight * tileset:getTileSize()
			
			local drawX, drawY = self:getPosition()
			local drawW, drawH = self:getScale()
			local drawHflip, drawVflip = self:getFlip()
			
			-- Rotoscale according to the offset
			love.graphics.translate(drawX, drawY)
			love.graphics.rotate(self:getAngle())
			love.graphics.scale((drawHflip == true) and -drawW or drawW,
								(drawVflip == true) and -drawH or drawH)
			love.graphics.translate(0 - drawX, 0 - drawY)
			
			-- Make the center of the tilegrid be the origin rather than the
			--  corner. This makes the behavior consistent with other types of
			--  Entities.
			love.graphics.translate(-0.5 * totalWidth + drawX, -0.5 * totalHeight + drawY)
			
			self._tilegrid:draw()
			
			love.graphics.pop()
		end
	end
end
