--[[
Copyright (c) 2013 Daniel Hagerstrand

This software is provided 'as-is', without any express or implied warranty. In
no event will the authors be held liable for any damages arising from the use
of this software.

Permission is granted to anyone to use this software for any purpose, including
commercial applications, and to alter it and redistribute it freely, subject to
the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software in a
product, an acknowledgment in the product documentation would be appreciated
but is not required.

    2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.

    3. This notice may not be removed or altered from any source distribution.
--]]


--[[
Cabin - A set of small logging helpers.
--]]

local _verbosityTable = {
	['debug'] = 0,
	['info'] = 1,
	['warning'] = 2,
	['error'] = 3
}

local _priv = {
	_observers = {},
	_verbosity = _verbosityTable['warning']
}

cabin = {}

local function _broadcast(message, severity)
	local severityNum = _verbosityTable[severity]
	
	if _priv._verbosity <= severityNum
				and type(message) == 'string' then
		for func in pairs(_priv._observers) do
			func(message)
		end
		
		if severityNum < _verbosityTable['error'] then
			print(message)
		else
			error(message)
		end
	end
end

function cabin.addObserver(callbackFunc)
	if type(callbackFunc) == 'function' then
		_priv._observers[callbackFunc] = true
	end
end

function cabin.setVerbosity(level)
	if type(level) == 'string' then
		_priv._verbosity = _verbosityTable[level] or _priv._verbosity
	end
end

function cabin.debug(message)
	_broadcast("Hmm: " .. message, 'debug')
end

function cabin.info(message)
	_broadcast(message, 'info')
end

function cabin.warning(message)
	_broadcast("Uh oh: " .. message, 'warning')
end

function cabin.error(message)
	_broadcast("Oops: " .. message, 'error')
end
