--[[
Copyright (c) 2013 Daniel Hagerstrand

This software is provided 'as-is', without any express or implied warranty. In
no event will the authors be held liable for any damages arising from the use
of this software.

Permission is granted to anyone to use this software for any purpose, including
commercial applications, and to alter it and redistribute it freely, subject to
the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software in a
product, an acknowledgment in the product documentation would be appreciated
but is not required.

    2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.

    3. This notice may not be removed or altered from any source distribution.
--]]

require "lib.klua"
require "lib.klutils.cabin"

Marker = Entity:makeClass
{
	_tics = 0
}

local _repeatDelay = 12
local _repeatRate = 4

function Marker:init(boardX, boardY, boardObj)
	if not boardObj then
		cabin.error("Marker must belong to a board object.")
	end
	
	self._boardX = boardX
	self._boardY = boardY
	
	self._boardObj = boardObj
	
	local originX, originY = boardObj:boardToScreenCoords(boardX, boardY)
	
	Entity.init(self, originX, originY)
	
	self:setCostume("marker")
end

function Marker:think()
	if not self._boardObj then
		return
	end
	
	local tilegrid = self._boardObj:getTilegrid()
	
	if not tilegrid then
		return
	end
	
	local width, height = tilegrid:getSize()
	local prevBoardX, prevBoardY = self._boardX, self._boardY
	local leftRepeat, righRepeat, upRepeat, downRepeat =
						input.autoRepeat('left', _repeatDelay, _repeatRate),
						input.autoRepeat('right', _repeatDelay, _repeatRate),
						input.autoRepeat('up', _repeatDelay, _repeatRate),
						input.autoRepeat('down', _repeatDelay, _repeatRate)
	local left, right, up, down = input.held('left'),
								input.held('right'),
								input.held('up'),
								input.held('down')
	
	if leftRepeat and not right and self._boardX > 0 then
		self._boardX = self._boardX - 1
	elseif righRepeat and not left and self._boardX < width - 1 then
		self._boardX = self._boardX + 1
	end
	if upRepeat and not down and self._boardY > 0 then
		self._boardY = self._boardY - 1
	elseif downRepeat and not up and self._boardY < height - 1 then
		self._boardY = self._boardY + 1
	end
	
	self:setPosition(self._boardObj:boardToScreenCoords(self._boardX, self._boardY))
	
	self._tics = self._tics + 0.05
	local scale = (1.0 - (0.2 * math.sin(self._tics)))
				* self._boardObj:getMarkerSize()
	
	self:setScale(scale, scale)
	
	self._boardObj:doObjectPlace(self._boardX, self._boardY)
end

function Marker:getBoardPosition()
	return self._boardX, self._boardY
end

function Marker:getParentBoard()
	return self._boardObj
end

function Marker:setParentBoard(board)
	if board then
		self._boardObj = board
	end
end
