--[[
Copyright (c) 2013 Daniel Hagerstrand

This software is provided 'as-is', without any express or implied warranty. In
no event will the authors be held liable for any damages arising from the use
of this software.

Permission is granted to anyone to use this software for any purpose, including
commercial applications, and to alter it and redistribute it freely, subject to
the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software in a
product, an acknowledgment in the product documentation would be appreciated
but is not required.

    2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.

    3. This notice may not be removed or altered from any source distribution.
--]]

require "lib.klua"
require "lib.klutils.cabin"

PuzzleBoard = TilegridEntity:makeClass
{
	
}

function PuzzleBoard:init(x, y, leveltable, levelnum, boardtype, tileset)
	if not leveltable then
		cabin.error("Not a level pack.")
	elseif not leveltable[levelnum] then
		cabin.error("Level number does not exist in level pack.")
	end
	
	local level = leveltable[levelnum]
	local tilesetName
	
	local tilegrid = Tilegrid:new(tileset)
	
	TilegridEntity.init(self, x, y, tilegrid)
	
	if not level[boardtype] then
		cabin.error("Bad level data.")
	end
	
	tilegrid:setTilesFromTable(level[boardtype])
	
end

function PuzzleBoard:think()

end

function PuzzleBoard:boardToScreenCoords(boardX, boardY)
	
	local originX, originY = self:getPosition()
	local sizeX, sizeY = self:getPosition()
	
	local tilegrid = self:getTilegrid()
	
	if not tilegrid then
		return nil
	end
	
	local sizeW, sizeH = tilegrid:getSize()
	
	local tileset = tilegrid:getTileset()
	
	if not tileset then
		return nil
	end
	
	local tilesize = tileset:getTileSize()
	
	sizeW = sizeW * tilesize
	sizeH = sizeH * tilesize
	
	originX = originX - sizeW / 2 + (boardX + 0.5) * tilesize
	originY = originY - sizeH / 2 + (boardY + 0.5) * tilesize
	
	return originX, originY
end
